package jadex.adapter.jade.tools.logger;

import jade.core.AID;
import jade.core.ContainerID;
import jade.core.event.JADEEvent;

import java.util.logging.Level;
import java.util.logging.LogRecord;


/**
 This class represents an log event.
 */
public class LogEvent extends JADEEvent
{

    public static final int PUBLISHED_LOGRECORD = 1;
    public static final int CHANGED_LOGLEVEL = 2;

    private AID agent;
    private Level levelfrom = null;
    private Level levelto = null;
    private LogRecord logRecord = null;

    public LogEvent(int id, AID aid, Level from, Level to, ContainerID cid)
    {
        super(id, cid);

        if (!isChangedLogLevel())
        {
            throw new InternalError("Bad Event kind: it must be a 'changed-loglevel' event.");
        }
        agent = aid;
        levelfrom = from;
        levelto = to;
        logRecord = null;
    }


    public LogEvent(int id, AID aid, LogRecord record, ContainerID cid)
    {
        super(id, cid);

        if (!isPublished())
        {
            throw new InternalError("Bad Event kind: it must be a 'published-logrecord' event.");
        }
        agent = aid;
        levelfrom = null;
        levelto = null;
        logRecord = record;
    }

    public AID getAgent()
    {
        return agent;
    }


    public Level getLevelfrom()
    {
        return levelfrom;
    }

    public Level getLevelto()
    {
        return levelto;
    }

    public LogRecord getLogRecord()
    {
        return logRecord;
    }

    private boolean isPublished()
    {
        return type == PUBLISHED_LOGRECORD;
    }

    private boolean isChangedLogLevel()
    {
        return type == CHANGED_LOGLEVEL;
    }


}
