package jadex.adapter.jade.tools.logger;

import jade.core.AID;
import jade.core.ContainerID;
import jade.core.Agent;
import jadex.adapter.jade.tools.logger.ILogListener;
import jadex.adapter.jade.tools.logger.LogEvent;

import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Level;


/**
 *  The log listener handler.
 */
public class LogListenerHandler extends Handler
{

	// the logged agent
	private AID agent;

	// listener for log events
	private ILogListener listener;

	// ContainerID for events
	private ContainerID container;


	/**
	 * Constructor for ACLMessagePreprocessor.
	 */
	public LogListenerHandler(ILogListener listener, AID agent)
	{
		this.agent = agent;
		this.listener = listener;
		this.container = (ContainerID) ((Agent) listener).here();
	}

	/**
	 * Constructor for ACLMessagePreprocessor.
	 */
	public LogListenerHandler(ILogListener listener, AID agent, ContainerID container)
	{
		this.agent = agent;
		this.listener = listener;
		this.container = container;
	}

	/**
	 * @see Handler#publish(LogRecord)
	 */
	public void publish(LogRecord record)
	{

		LogEvent le = new LogEvent(LogEvent.PUBLISHED_LOGRECORD, agent, record, container);
		listener.publishedLogRecord(le);
	}

	/**
	 * @see Handler#flush()
	 */
	public void flush()
	{
	}

	/**
	 * @see Handler#close()
	 */
	public void close() throws SecurityException
	{
	}

	/**
	 *  Set the logging level.
	 *  @param newLevel
	 *  @throws SecurityException
	 */
	public synchronized void setLevel(Level newLevel) throws SecurityException
	{
		LogEvent le = new LogEvent(LogEvent.CHANGED_LOGLEVEL, agent, getLevel(), newLevel, container);
		listener.changedLogLevel(le);

		super.setLevel(newLevel);
	}

	/**
	 *  Get the listener.
	 *  @return The listener.
	 */
	public ILogListener getListener()
	{
		return listener;
	}

}
