package jadex.adapter.jade.tools.logger;

import java.util.Map;
import java.util.logging.*;
import java.util.logging.Logger;
import jade.content.*;
import jade.content.abs.*;
import jade.content.lang.Codec;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.*;
import jade.core.*;
import jade.core.behaviours.*;
import jade.domain.FIPAAgentManagement.UnsupportedFunction;
import jade.domain.*;
import jade.domain.introspection.*;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.*;
import jade.tools.ToolAgent;
import jade.util.leap.*;
import jadex.adapter.jade.tools.logger.ontology.*;

public class LoggerNotifier extends ToolAgent implements ILogListener
{


    private HashMap observedAgents = new HashMap();
    private SequentialBehaviour AMSSubscribe = new SequentialBehaviour();
    private ACLMessage msg = new ACLMessage(ACLMessage.INFORM);

    // flag used to stop handling events during shutdown
    private boolean closingDown = false;

    private final Codec SL0 = new SLCodec(0);

    private AID observerAgent;


    /**
     Inner class NotifierAMSListenerBehaviour
     */
    class NotifierAMSListenerBehaviour extends AMSListenerBehaviour
    {

        protected void installHandlers(Map handlersTable)
        {
            // Fill the event handler table.
            handlersTable.put(IntrospectionOntology.DEADAGENT, new EventHandler()
            {
                public void handle(Event ev)
                {
                    DeadAgent da = (DeadAgent) ev;
                    AID dead = da.getAgent();
                    removeObservedAgent(dead);
                }
            });

            handlersTable.put(IntrospectionOntology.MOVEDAGENT, new EventHandler()
            {
                public void handle(Event ev)
                {
                    MovedAgent ma = (MovedAgent) ev;
                    AID moved = ma.getAgent();
                    if (!here().equals(ma.getTo()))
                    {
                        removeObservedAgent(moved);
                    }
                }
            });

        } // END of installHandlers() method

    } // END of inner class NotifierAMSListenerBehaviour

    class ObserverRequestsHandler extends RequestManagementBehaviour
    {

        public ObserverRequestsHandler(Agent a, MessageTemplate mt)
        {
            super(a, mt);
        }

        protected ACLMessage performAction(Action slAction, ACLMessage request) throws FIPAException
        {
            Concept action = slAction.getAction();
            Object result = null;
            boolean resultNeeded = false;

            // LOG ON
            if (action instanceof LogOn)
            {
                logOnAction((LogOn) action, request.getSender());
            }
            // LOG OFF
            else if (action instanceof LogOff)
            {
                logOffAction((LogOff) action, request.getSender());
            }
            else
            {
                throw new UnsupportedFunction();
            }

            // Prepare the notification
            ACLMessage notification = request.createReply();
            notification.setPerformative(ACLMessage.INFORM);
            Predicate p = null;
            if (resultNeeded)
            {
                // The action produced a result
                p = new Result(slAction, result);
            }
            else
            {
                p = new Done(slAction);
            }
            try
            {
                getContentManager().fillContent(notification, p);
            }
            catch (Exception e)
            {
                // Should never happen
                e.printStackTrace();
            }

            return notification;
        }
    }


    protected void toolSetup()
    {
        //        getContentManager().registerLanguage(SL0);
        getContentManager().registerOntology(LoggerOntology.getInstance());

        // load observerAgent from Arguments
        // decode sl-string argument to aid
        String arguments = getArguments()[0].toString();
        SLCodec sc = new SLCodec();
        try
        {
            AbsPredicate pred = (AbsPredicate) sc.decode(arguments);
            AbsObject obj = pred.getAbsTerm("_JADE.UNNAMED0");
            observerAgent = AbsHelper.internaliseAID((AbsConcept) obj);
        }
        catch (Codec.CodecException e)
        {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }
        catch (OntologyException e)
        {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }


        // Send 'subscribe' message to the AMS
        AMSSubscribe.addSubBehaviour(new SenderBehaviour(this, getSubscribe()));

        // Handle incoming 'inform' messages from the AMS
        AMSSubscribe.addSubBehaviour(new NotifierAMSListenerBehaviour());
        addBehaviour(AMSSubscribe);

        // Handle requests from the observer agent
        MessageTemplate mt1 = MessageTemplate.MatchPerformative(ACLMessage.REQUEST);
        MessageTemplate mt2 = MessageTemplate.MatchOntology(LoggerOntology.ONTOLOGY_NAME);
        MessageTemplate mt = MessageTemplate.and(mt1, mt2);
        addBehaviour(new ObserverRequestsHandler(this, mt));

        // Set constant fields in the message to be sent to the observer each
        // time an event occurs.
        msg.setSender(getAID());
        msg.addReceiver(observerAgent);
        msg.setLanguage(FIPANames.ContentLanguage.FIPA_SL0);

    }

    protected void toolTakeDown()
    {
        closingDown = true;

        Iterator iterator = observedAgents.keySet().iterator();
        while (iterator.hasNext())
        {
            AID agent = (AID) iterator.next();

            Logger logger = Logger.getLogger(agent.getName());
            LogListenerHandler handler = (LogListenerHandler) observedAgents.get(agent);
            logger.removeHandler(handler);

            StopNotify sn = new StopNotify();
            sn.setObserved(agent);
            // cant youse DoneInformer behaviour, because after doDelete
            // no behaviours are scheduled anymore
            Action a = new Action(getAID(), sn);
            Done d = new Done(a);

            try
            {
                msg.setConversationId(observerAgent.getName() + "-control");
                msg.setOntology(IntrospectionOntology.NAME);
                getContentManager().fillContent(msg, d);
            }
            catch (Codec.CodecException e)
            {
                e.printStackTrace();
            }
            catch (OntologyException e)
            {
                e.printStackTrace();
            }
            send(msg);
        }

        observedAgents.clear();

        send(getCancel());
    }

    private void logOnAction(LogOn logOn, AID sender)
    {
        AID[]	logged	= logOn.getLoggedAgents();
        for(int i=0; i<logged.length; i++)
        {
            addObservedAgent(logged[i]);

        }
    }

    private void logOffAction(LogOff logOff, AID sender)
    {
        AID[]	logged	= logOff.getLoggedAgents();
        for(int i=0; i<logged.length; i++)
        {
            removeObservedAgent(logged[i]);
        }

    }

    public void addObservedAgent(AID id)
    {
        Logger logger = Logger.getLogger(id.getName());
        LogListenerHandler handler = new LogListenerHandler(this, id);        
        logger.addHandler(handler);

        observedAgents.put(id, handler);

        StartNotify sn = new StartNotify();
        sn.setObserved(id);
        addBehaviour(new DoneInformer(this, sn));
    }

    public void removeObservedAgent(AID id)
    {
        if (observedAgents.containsKey(id))
        {
            Logger logger = Logger.getLogger(id.getName());
            LogListenerHandler handler = (LogListenerHandler) observedAgents.get(id);
            logger.removeHandler(handler);

            observedAgents.remove(id);

            StopNotify sn = new StopNotify();
            sn.setObserved(id);
            addBehaviour(new DoneInformer(this, sn)
            {
                // Must call doDelete onEnd to ensure
                // that this behaviour is scheduled
                public int onEnd()
                {
                    if (isEmpty())
                    {
                        doDelete();
                    }
                    return super.onEnd();
                }
            });

        }
    }


    public boolean isEmpty()
    {
        return observedAgents.isEmpty();
    }


    /////////////////////////////////////
    // ILogListener Interface
    /////////////////////////////////////
    public void publishedLogRecord(LogEvent ev)
    {
        if (closingDown)
            return;

        AID id = ev.getAgent();

        LogRecord record = ev.getLogRecord();

        // Create Ontology-Concept from LogRecord
        PublishedLogRecord plr = new PublishedLogRecord();
        plr.setLevel(record.getLevel().getLocalizedName());
        plr.setLoggerName(record.getLoggerName());
        plr.setMessage(record.getMessage());
        plr.setMillis(new Long(record.getMillis()).toString());
        plr.setSequenceNumber(new Long(record.getSequenceNumber()).toString());
        plr.setSourceClassName(record.getSourceClassName());
        plr.setSourceMethodName(record.getSourceMethodName());
        plr.setThreadID(record.getThreadID());

        addBehaviour(new EventInformer(this, plr));


    }

    public void changedLogLevel(LogEvent ev)
    {
        if (closingDown)
            return;

        AID id = ev.getAgent();

        Level from = ev.getLevelfrom();
        Level to = ev.getLevelto();

        ChangedLogLevel cll = new ChangedLogLevel();
        cll.setAgent(id);
        cll.setFrom(from.getLocalizedName());
        cll.setTo(to.getLocalizedName());

        addBehaviour(new EventInformer(this, cll));


    }

    /**
     Inner class DoneInformer.
     Inform the Observer that an action (typically StartNotify)
     has been done.
     */
    private class DoneInformer extends OneShotBehaviour
    {
        private Object act;

        DoneInformer(Agent a, Object act)
        {
            super(a);
            this.act = act;
        }

        public void action()
        {
            Action a = new Action();
            a.setAction((AgentAction) act);
            a.setActor(getAID());
            Done d = new Done();
            d.setAction(a);
            try
            {
                msg.setConversationId(observerAgent.getName() + "-control");
                msg.setOntology(IntrospectionOntology.NAME);
                getContentManager().fillContent(msg, d);
                send(msg);
            }
            catch (Exception fe)
            {
                fe.printStackTrace();
            }
        }
    }

    /**
     Inner class EventInformer.
     Inform the Observer about an event that has occurred
     */
    private class EventInformer extends OneShotBehaviour
    {
        private Event ev;

        EventInformer(Agent a, Event ev)
        {
            super(a);
            this.ev = ev;
            setBehaviourName(getBehaviourName() + "-" + ev.toString());
        }

        public void action()
        {
            EventRecord er = new EventRecord(ev, here());
            Occurred o = new Occurred();
            o.setWhat(er);

            try
            {
                msg.setConversationId(observerAgent.getName() + "-event");
                msg.setOntology(LoggerOntology.ONTOLOGY_NAME);
                getContentManager().fillContent(msg, o);
                send(msg);
            }
            catch (Exception fe)
            {
                fe.printStackTrace();
            }
        }
    }


}
