package jadex.adapter.jade.tools.logger;

import jade.content.abs.AbsHelper;
import jade.content.abs.AbsObject;
import jade.content.abs.AbsPredicate;
import jade.content.lang.Codec;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.content.onto.basic.Result;
import jade.core.AID;
import jade.core.ContainerID;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.FSMBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.FIPAAgentManagement.*;
import jade.domain.FIPANames;
import jade.domain.RequestFIPAServiceBehaviour;
import jade.domain.JADEAgentManagement.CreateAgent;
import jade.domain.JADEAgentManagement.JADEManagementOntology;
import jade.domain.JADEAgentManagement.WhereIsAgentAction;
import jade.lang.acl.ACLMessage;
import jade.proto.SimpleAchieveREInitiator;
import jade.util.leap.Iterator;
import jadex.adapter.jade.tools.logger.ontology.LogOff;
import jadex.adapter.jade.tools.logger.ontology.LogOn;
import jadex.adapter.jade.tools.logger.ontology.LoggerOntology;

/**
 *
 */
public class RequestLoggingBehaviour extends FSMBehaviour
{

    protected final String INITIATION_K = "__initiation" + hashCode();
    protected final String ALL_INITIATIONS_K = "__all-initiations" + hashCode();
    protected final String REPLY_K = "__reply" + hashCode();

    private static final int NEED_NOTHING = 0;
    private static final int NEED_NOTIFIER = 1;
    private static final int NEED_CONTAINERID = 2;
    private static final int NEED_LOGREQUEST = 3;


    // FSM states names
    protected static final String CHECK_NEEDS = "Check-Needs";
    protected static final String GET_CONTAINERID = "Get-ContainerId";
    protected static final String GET_NOTIFIER = "Get-Notifier";
    protected static final String SEARCH_NOTIFIER = "Search-Notifier";
    protected static final String CREATE_NOTIFIER = "Create-Notifier";
    protected static final String SEND_LOGREQUEST = "Send-LogRequest";
    protected static final String SEND_LOGON = "Send-LogOn";
    protected static final String SEND_LOGOFF = "Send-LogOff";
    protected static final String DUMMY_FINAL = "Dummy-final";

    private long timeout = 5000; // 5sec

    private Logger myLogger;

    private AID notifier;
    private AID toBeLogged;
    private AID ams;
    private ContainerID cid;

    /**
     This constructor sets the owner agent for this behaviour.
     @param a The agent this behaviour belongs to.
     @param toBeLogged The agentidentifier of the agent to be logged
     */
    public RequestLoggingBehaviour(Logger myLogger, final AID toBeLogged, final ContainerID cid, final AID ams, final AID notifierID, final boolean onFlag)
    {
        super(myLogger);

        this.myLogger = myLogger;
        this.toBeLogged = toBeLogged;
        this.ams = ams;
        this.notifier = notifierID;
        this.cid = cid;

        // Register the FSM transitions
        registerDefaultTransition(CHECK_NEEDS, DUMMY_FINAL);
        registerTransition(CHECK_NEEDS, GET_CONTAINERID, NEED_CONTAINERID);
        registerTransition(CHECK_NEEDS, GET_NOTIFIER, NEED_NOTIFIER);
        registerTransition(CHECK_NEEDS, SEND_LOGREQUEST, NEED_LOGREQUEST);
        registerDefaultTransition(GET_CONTAINERID, GET_NOTIFIER);
        registerDefaultTransition(GET_NOTIFIER, SEARCH_NOTIFIER);
        //        registerDefaultTransition(GET_NOTIFIER, CREATE_NOTIFIER);
        registerTransition(SEARCH_NOTIFIER, CREATE_NOTIFIER,NEED_NOTIFIER);
        registerDefaultTransition(SEARCH_NOTIFIER, SEND_LOGREQUEST);
        registerDefaultTransition(CREATE_NOTIFIER, SEND_LOGREQUEST);
        registerDefaultTransition(SEND_LOGREQUEST, DUMMY_FINAL);

        // Create and register the states that make up the FSM
        Behaviour b = null;
        // CHECK_NEEDS
        b = new OneShotBehaviour(myAgent)
        {
            private static final long serialVersionUID = 3487495895818000L;
            private int ret = 0;

            public void action()
            {
                if (notifier == null && onFlag)
                {
                    if (cid == null)
                    {
                        ret = NEED_CONTAINERID;
                    }
                    else
                    {
                        ret = NEED_NOTIFIER;
                    }
                }
                else if (notifier != null && !onFlag)
                {
                    ret = NEED_LOGREQUEST;
                }
            }

            public int onEnd()
            {
                return ret;
            }
        };
        b.setDataStore(getDataStore());
        registerFirstState(b, CHECK_NEEDS);

        // GET_CONTAINERID
        b = new GetContainerIDBehaviour();
        registerState(b, GET_CONTAINERID);

        // GET_NOTIFIER
        b = new CreateNotifierAIDBehaviour();
        registerState(b, GET_NOTIFIER);

        // SEARCH_NOTIFIER
        b = new SearchNotifierBehaviour();
//        b=new RequestFIPAServiceBehaviour(myLogger,ams,FIPAManagementVocabulary.SEARCH,);
        registerState(b, SEARCH_NOTIFIER);

        // CREATE_NOTIFIER
        b = new CreateNotifierAgentBehaviour();
        registerState(b, CREATE_NOTIFIER);

        // SEND_LOGREQUEST
        b = new AMSClientBehaviour(onFlag ? "LogAgentOn" : "LogAgentOff", null)
        {
            protected ACLMessage prepareRequest(ACLMessage msg)
            {
                return getLogMsg(toBeLogged, notifier, onFlag);
            }


        };
        registerState(b, SEND_LOGREQUEST);

        // DUMMY_FINAL
        b = new OneShotBehaviour(myAgent)
        {
            private static final long serialVersionUID = 3487495895818010L;

            public void action()
            {
            }
        };
        registerLastState(b, DUMMY_FINAL);


    }


    private ACLMessage getJadeRequest()
    {
        ACLMessage request = new ACLMessage(ACLMessage.REQUEST);
        request.setSender(myAgent.getAID());
        request.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
        request.setLanguage(FIPANames.ContentLanguage.FIPA_SL0);
        request.setOntology(JADEManagementOntology.NAME);
        return request;
    }

    private ACLMessage getFipaRequest()
    {
        ACLMessage request = new ACLMessage(ACLMessage.REQUEST);
        request.setSender(myAgent.getAID());
        request.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
        request.setLanguage(FIPANames.ContentLanguage.FIPA_SL0);
        request.setOntology(FIPAManagementOntology.NAME);
        return request;
    }

    private ACLMessage getNotifierRequest()
    {
        ACLMessage request = new ACLMessage(ACLMessage.REQUEST);
        request.setSender(myAgent.getAID());
        request.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
        request.setLanguage(FIPANames.ContentLanguage.FIPA_SL0);
        request.setOntology(LoggerOntology.ONTOLOGY_NAME);
        request.setConversationId(myAgent.getName() + "-request");
        return request;
    }


    private ACLMessage getContainerMsg(AID agent, AID ams)
    {
        ACLMessage requestMsg = getJadeRequest();
        requestMsg.addReceiver(ams);

        WhereIsAgentAction wia = new WhereIsAgentAction();
        wia.setAgentIdentifier(agent);

        Action a = new Action();
        a.setActor(myAgent.getAID());
        a.setAction(wia);

        try
        {
            myAgent.getContentManager().fillContent(requestMsg, a);
        }
        catch (OntologyException e)
        {
            e.printStackTrace();
        }
        catch (Codec.CodecException e)
        {
            e.printStackTrace();
        }
        return requestMsg;
    }

    private ACLMessage searchNotifierMsg(AID ams, ContainerID cid)
    {
        ACLMessage request = getFipaRequest();
        request.addReceiver(ams);

        SearchConstraints constraints = new SearchConstraints();
        AMSAgentDescription amsd = new AMSAgentDescription();
        amsd.setName(notifier);

        Search s = new Search();
        s.setDescription(amsd);
        s.setConstraints(constraints);

        Action a = new Action();
        a.setActor(myAgent.getAID());
        a.setAction(s);

        try
        {
            myAgent.getContentManager().fillContent(request, a);
        }
        catch (Exception fe)
        {
            fe.printStackTrace();
        }
        return request;
    }


    private ACLMessage createNotifierMsg(AID ams, ContainerID cid)
    {
        ACLMessage request = getJadeRequest();
        request.addReceiver(ams);

        String classNotifier = LoggerNotifier.class.getName();
        String nameContainer = cid.getName();
        String nameNotifier = notifier.getLocalName();
        String identifier = null;

        // encode aid of logger to sl-string.
        AbsObject obj = AbsHelper.externaliseAID(myAgent.getAID());
        AbsPredicate pred = new AbsPredicate("OBSERVER-AID");
        pred.set("AID", obj);
        SLCodec sl = new SLCodec();
        try
        {
            identifier = sl.encode(pred);
        }
        catch (Codec.CodecException e)
        {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }

        CreateAgent ca = new CreateAgent();
        ca.setAgentName(nameNotifier);
        ca.setClassName(classNotifier);
        ca.setContainer(new ContainerID(nameContainer, null));
        ca.addArguments(identifier);


        Action a = new Action();
        a.setActor(myAgent.getAMS());
        a.setAction(ca);

        try
        {
            myAgent.getContentManager().fillContent(request, a);
        }
        catch (Exception fe)
        {
            fe.printStackTrace();
        }

        return request;
    }


    public ACLMessage getLogMsg(AID agent, AID notifier, boolean onFlag)
    {
        // create ACLMessage for logging/unlogging agent
        ACLMessage request = getNotifierRequest();
        request.addReceiver(notifier);

        if (onFlag)
        {
            LogOn lo = new LogOn();
            lo.setLogger(myAgent.getAID());
            lo.addLoggedAgent(agent);
            try
            {
                Action a = new Action(myAgent.getAID(), lo);
                myAgent.getContentManager().fillContent(request, a);
                return request;
            }
            catch (Exception fe)
            {
                fe.printStackTrace();
            }
        }
        else
        {
            LogOff lo = new LogOff();
            lo.setLogger(myAgent.getAID());
            lo.addLoggedAgent(agent);
            try
            {
                Action a = new Action(myAgent.getAID(), lo);
                myAgent.getContentManager().fillContent(request, a);
                return request;
            }
            catch (Exception fe)
            {
                fe.printStackTrace();
            }
        }
        return null;
    }

    /*
     * Sends requests to the AMS for platform dependent actions
     * e.g. create LoggerNotifier
     */
    private class AMSClientBehaviour extends SimpleAchieveREInitiator
    {

        private String actionName;

        public AMSClientBehaviour(String an, ACLMessage request)
        {
            super(myLogger, request);
            actionName = an;
        }

        protected void handleNotUnderstood(ACLMessage reply)
        {
            myLogger.myGUI.showError("NOT-UNDERSTOOD received during " + actionName);
        }

        protected void handleRefuse(ACLMessage reply)
        {
            myLogger.myGUI.showError("REFUSE received during " + actionName);
        }

        protected void handleAgree(ACLMessage reply)
        {
            // System.out.println("AGREE received");
        }

        protected void handleFailure(ACLMessage reply)
        {
            myLogger.myGUI.showError("FAILURE received during " + actionName);
            //            myLogger.removeBehaviour(LoggerNotifierBehaviour.this);
        }

        protected void handleInform(ACLMessage reply)
        {
            // System.out.println("INFORM received");
        }

    } // End of AMSClientBehaviour class


    private class GetContainerIDBehaviour extends AMSClientBehaviour
    {
        public GetContainerIDBehaviour()
        {
            super("GetContainer", null);
        }

        protected ACLMessage prepareRequest(ACLMessage msg)
        {
            return getContainerMsg(toBeLogged, ams);
        }

        public int onEnd()
        {
            return super.onEnd();
        }

        protected void handleInform(ACLMessage inform)
        {
            try
            {
                Result r = (Result) myAgent.getContentManager().extractContent(inform);
                cid = (ContainerID) r.getValue();
            }
            catch (Codec.CodecException e)
            {
                e.printStackTrace();
            }
            catch (OntologyException e)
            {
                e.printStackTrace();
            }
        }
    }

    private class SearchNotifierBehaviour extends AMSClientBehaviour
    {
        private int ret=0;

        public SearchNotifierBehaviour()
        {
            super("SearchNotifier", null);
        }

        protected ACLMessage prepareRequest(ACLMessage msg)
        {
            return searchNotifierMsg(ams, cid);
        }

        protected void handleInform(ACLMessage reply)
        {
            try
            {
                Result r = (Result) myAgent.getContentManager().extractContent(reply);
                if (r.getItems().size() == 0)
                {
                    // if the logger already exists, skip creating of
                    // LoggerNotifierBehaviour.this.skipNext();
                    ret = NEED_NOTIFIER;
                }
            }
            catch (Codec.CodecException e)
            {
                e.printStackTrace();
            }
            catch (OntologyException e)
            {
                e.printStackTrace();
            }
        }

        public int onEnd()
        {
            return ret;
        }
    }

    private class CreateNotifierAgentBehaviour extends AMSClientBehaviour
    {
        public CreateNotifierAgentBehaviour()
        {
            super("CreateNotifier", null);
        }

        protected ACLMessage prepareRequest(ACLMessage msg)
        {
            return createNotifierMsg(ams, cid);
        }

        protected void handleFailure(ACLMessage reply)
        {
            // prevents showing error message if agent is already registered
            // FIXME: Show message for the other errors
        }
    }

    private class CreateNotifierAIDBehaviour extends OneShotBehaviour
    {
        public CreateNotifierAIDBehaviour()
        {
            super(myLogger);
        }

        public int onEnd()
        {
            return super.onEnd();
        }

        public void action()
        {
            String nameNotifier;
            if (ams.equals(myAgent.getAMS()))
            {
                nameNotifier = myAgent.getLocalName() + "-on-" + cid.getName();
            }
            else
            {
                nameNotifier = myAgent.getLocalName() + "-on-" + cid.getName() + "-from-" + myAgent.getHap();
            }

            String hapNotifier = ams.getName().substring(ams.getLocalName().length() + 1);
            notifier = new AID(nameNotifier + "@" + hapNotifier, AID.ISGUID);
            Iterator addresses = ams.getAllAddresses();
            while (addresses.hasNext())
            {
                notifier.addAddresses((String) addresses.next());
            }
            Iterator resolvers = ams.getAllResolvers();
            while (resolvers.hasNext())
            {
                notifier.addResolvers((AID) resolvers.next());
            }
        }
    }

}
