/*****************************************************************
 JADE - Java Agent DEvelopment Framework is a framework to develop
 multi-agent systems in compliance with the FIPA specifications.
 Copyright (C) 2000 CSELT S.p.A.

 GNU Lesser General Public License

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation,
 version 2.1 of the License.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the
 Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.
 *****************************************************************/

package jadex.adapter.jade.tools.logger.gui;

import java.util.Map;
import java.util.HashMap;
import java.awt.event.ActionEvent;

import jade.gui.AgentTree;
import jade.tools.sniffer.*;

import javax.swing.*;
import javax.swing.tree.TreePath;

import jadex.adapter.jade.tools.logger.Logger;
import jadex.adapter.jade.tools.rma.*;

/**
 Javadoc documentation for the file
 @author Francisco Regi, Andrea Soracchi - Universita` di Parma
 <Br>
 <a href="mailto:a_soracchi@libero.it"> Andrea Soracchi(e-mail) </a>
 @version $Date: 2005-11-04 12:44:04 +0000 (Fri, 04 Nov 2005) $ $Revision: 3404 $
 */

 class ActionProcessor
{

    public static final String CLEARTABLE_ACTION = "ClearTableAction";
    public static final String TOGGLEFILTER_ACTION = "ToggleFilter";
    public static final String EDITFILTER_ACTION = "EditFilter";
    public static final String REMOVEFILTER_ACTION = "RemoveFilter";

    public static final String TOGGLEDISPLAYING_ACTION = "ToggleDisplaying";
    public static final String TOGGLEAUTOSCROLL_ACTION = "ToggleAutoscroll";
    public static final String RESIZEALLCOLUMNS_ACTION = "ResizeAllColumns";


    public static final String DISPLAYLOGFILE_ACTION = "DisplayLogFileAction";
    public static final String WRITELOGFILE_ACTION = "WriteLogFileAction";
    public static final String WRITEMESSAGELIST_ACTION = "MessageFileAction";
    public static final String EXIT_LOGGER_ACTION = "ExitLoggerAction";
    public static final String PRELOAD_LIST_ACTION = "PreloadListAction";
    public static final String DO_LOGGER_ACTION = "DoLoggerAction";
    public static final String DO_NOT_LOGGER_ACTION = "DoNotLoggerAction";
    public static final String SWOW_ONLY_ACTION = "ShowOnlyAction";
    public static final String ADDREMOTEPLATFORM_ACTION = "AddRemotePlatformAction";
    // public static final String  ADDREMOTEPLATFORMFROMURL_ACTION = "Add Platform via URL";
    public static final String REFRESHAMSAGENT_ACTION = "RefreshAMSAgent";


    public static final Map actions = new HashMap(5);
    private MainPanel mainPanel;
    private MainWindow mainWindow;
    private LoggerAction action;

    public ActionProcessor(Logger myLogger, MainWindow mainWindow, MainPanel mainPanel)
    {
        this.mainPanel = mainPanel;
        this.mainWindow =mainWindow;
        actions.put(CLEARTABLE_ACTION, new ClearTableAction(this, mainPanel));
        actions.put(TOGGLEFILTER_ACTION, new ToggleFilterAction(this, mainPanel));
        actions.put(EDITFILTER_ACTION, new EditFilterAction(this, mainPanel));
        actions.put(REMOVEFILTER_ACTION, new RemoveFilterAction(this, mainPanel));
        actions.put(TOGGLEDISPLAYING_ACTION, new ToggleDisplayingAction(this, mainPanel));
        actions.put(TOGGLEAUTOSCROLL_ACTION, new ToggleAutoscrollAction(this, mainPanel));
        actions.put(RESIZEALLCOLUMNS_ACTION, new ResizeAllColumnsAction(this, mainPanel));
        actions.put(DISPLAYLOGFILE_ACTION, new DoLoggerAction(this, mainPanel, myLogger));
        actions.put(WRITELOGFILE_ACTION, new WriteLogFileAction(this, mainPanel));
        actions.put(WRITEMESSAGELIST_ACTION, new DoLoggerAction(this, mainPanel, myLogger));
        actions.put(PRELOAD_LIST_ACTION, new PreloadListAction(this, mainPanel,myLogger));
        actions.put(EXIT_LOGGER_ACTION, new ExitAction(this, myLogger));
        actions.put(DO_LOGGER_ACTION, new DoLoggerAction(this, mainPanel, myLogger));
        actions.put(DO_NOT_LOGGER_ACTION, new DoNotLoggerAction(this, mainPanel, myLogger));
        actions.put(SWOW_ONLY_ACTION, new DoLoggerAction(this, mainPanel, myLogger));
        actions.put(ADDREMOTEPLATFORM_ACTION, new AddRemotePlatformAction(myLogger, this, mainPanel));
//    actions.put(ADDREMOTEPLATFORMFROMURL_ACTION, new AddRemotePlatformFromURLAction(anRma,this,mWnd));
        actions.put(REFRESHAMSAGENT_ACTION, new RefreshAMSAgentAction(myLogger, this));

    }

    public void process(ActionEvent avt, LoggerAction act)
    {
        int lungpath;
        AgentTree.Node now;
//  FixedAction fx;
        TreePath paths[];
//  AgentAction agentAction;
        action = act;
        paths = mainPanel.treeAgent.tree.getSelectionPaths();

// Fixed actions are without parameters, so they are executed once,
// regardless how many tree elements are selected

        if (action instanceof FixedAction)
        {
            fixedAct();
        }
        else if (action instanceof ButtonAction)
        {
            AbstractButton button = (AbstractButton) avt.getSource();
            buttonAct(button);
        }

// Other actions are executed for every selected tree element. This
// means that, if no selection is present, no action is performed.
        else
        {
            if (paths != null)
            {
                lungpath = paths.length;
                for (int i = 0; i < lungpath; i++)
                {
                    now = (AgentTree.Node) (paths[i].getLastPathComponent());
                    if (action instanceof AgentAction)
                    {
                        agentAct(now);
                    }
                    else if (action instanceof PlatformAction)
                    {
                        platformAct(now);
                    }
                }
            }
        }

    } // End Process

    private void fixedAct()
    {
        FixedAction fx = (FixedAction) action;
        fx.doAction();
    }

    private void buttonAct(AbstractButton button)
    {
        ButtonAction bu = (ButtonAction) action;
        bu.doAction(button);
    }

    private void agentAct(AgentTree.Node node)
    {
        AgentAction ag = (AgentAction) action;
        AgentTree.AgentNode nod;
        try
        {

            if (node instanceof AgentTree.AgentNode)
            {
                nod = (AgentTree.AgentNode) node;
                ag.doAction(nod);
            }

            else
                throw new StartException();

        }
        catch (StartException a)
        {
            a.handle();
        }
    }

    private void platformAct(AgentTree.Node node)
    {

        PlatformAction ac = (PlatformAction) action;
//        if ((node instanceof AgentTree.localPlatformFolderNode) || (node instanceof AgentTree.RemotePlatformNode))
        // Hack!!! Class has been renamed to LocalPlatformFolderNode in latest jade
        if ((node.getClass().getName().indexOf("ocalPlatformFolderNode")!=-1) || (node instanceof AgentTree.RemotePlatformNode))
            ac.doAction(node);

    }

}