/*****************************************************************
 JADE - Java Agent DEvelopment Framework is a framework to develop
 multi-agent systems in compliance with the FIPA specifications.
 Copyright (C) 2000 CSELT S.p.A.

 GNU Lesser General Public License

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation,
 version 2.1 of the License.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the
 Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.
 *****************************************************************/

package jadex.adapter.jade.tools.logger.gui;

import javax.swing.tree.MutableTreeNode;
import java.awt.*;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.SwingUtilities;
import java.net.InetAddress;
import java.util.Iterator;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;

import jade.gui.AgentTree;
import jade.core.AID;
import jade.util.BasicProperties;
import jade.domain.FIPAAgentManagement.APDescription;
import jade.domain.FIPAAgentManagement.AMSAgentDescription;
import jadex.adapter.jade.tools.logger.Logger;
import jadex.adapter.jade.tools.logger.gui.ActionProcessor;
import jadex.adapter.jade.tools.logger.gui.MainMenu;
import jadex.adapter.jade.tools.logger.gui.MainPanel;


/**
 Javadoc documentation for the file
 @author Francisco Regi, Andrea Soracchi - Universita` di Parma
 <Br>
 <a href="mailto:a_soracchi@libero.it"> Andrea Soracchi(e-mail) </a>
 @version $Date: 2005-04-26 13:45:51 +0000 (Tue, 26 Apr 2005) $ $Revision: 2556 $
 */

/**
 * This class performs the <em>Sniffer</em> main-windows GUI setup. Also provides method for
 * asynchronous disposal at takedown.
 *
 * @see	javax.swing.JFrame
 */


public class MainWindow extends JFrame
{

    public MainPanel mainPanel;
    protected ToolBar toolBar;
    public ActionProcessor actPro; // Making this public allows us to get directly to the sniff agent action.
    private PopupMenuAgent popA;
    private PopupMenuPlatform popM;
    private PopupMenuRemotePlatform popRP;
    private Logger myLogger;
    private BasicProperties myProperties;
    private String loggerLogo = "images/logger.png";

    public MainWindow(Logger myLogger, BasicProperties myProperties)
    {
        super(myLogger.getName() + " - Logger Agent");
        this.myLogger = myLogger;
        this.myProperties = myProperties;
        mainPanel = new MainPanel(myLogger, this);
        actPro = new ActionProcessor(myLogger, this, mainPanel);
        toolBar = new ToolBar(actPro);
        setJMenuBar(new MainMenu(this, actPro));
        popA = new PopupMenuAgent(actPro);
        popM = new PopupMenuPlatform(actPro);
        popRP = new PopupMenuRemotePlatform(actPro);

        setForeground(Color.black);
        setBackground(Color.lightGray);
        Image image = getToolkit().getImage(getClass().getResource(loggerLogo));
        setIconImage(image);

        addWindowListener(new WindowCloser(myLogger));
        mainPanel.treeAgent.register("FIPAAGENT", popA, "images/runtree.gif");
        mainPanel.treeAgent.register("FIPACONTAINER", null, "images/foldergreen.gif");
        mainPanel.treeAgent.register("REMOTEPLATFORM", popRP, "images/folderlightblue.gif");
        mainPanel.treeAgent.register("REMOTEAGENT", popA, "images/runtree.gif");

        getContentPane().add(toolBar, "North");
        getContentPane().add(mainPanel, "Center");


        // set default properties
        boolean enableFilter = myProperties.getBooleanProperty("EnableFilter", true);
        mainPanel.logTable.setDoFilter(enableFilter);
        toolBar.enableFilter.setSelected(enableFilter);

        boolean showFilter = myProperties.getBooleanProperty("ShowFilter", true);
        mainPanel.setFilterVisible(showFilter);
        toolBar.showFilter.setSelected(showFilter);

//        BasicProperties filter_props = myProperties.extractSubset("filter_");
//        Enumeration enum=filter_props.keys();
//        Vector vec=mainPanel.logTable.getFilter();
//        while (enum.hasMoreElements()) {
//            String key=(String) enum.nextElement();
//            String filter = myProperties.getProperty("filter_" + key);
//            StringTokenizer st = new StringTokenizer(filter,";");
//            Vector row=new Vector();
//            while (st.hasMoreTokens()) {
//                row.addElement(st.nextToken());
//            }
//            vec.addElement(row);
//        }



    }


    public void ShowCorrect()
    {
        pack();

        // get dimensions from properties
        int width = myProperties.getIntProperty("Width", 700);
        int height = myProperties.getIntProperty("Height", 500);
        int x = myProperties.getIntProperty("LocationX", 0);
        int y = myProperties.getIntProperty("LocationY", 0);
        setSize(new Dimension(width, height));
        setLocation(new Point(x, y));

        mainPanel.adjustDividerLocation();
        this.setVisible(true);
        toFront();
    }

    public BasicProperties getProperties()
    {
        return myProperties;
    }

    /* since snapshot 030922
    public void resetTree() {
        Runnable resetIt = new Runnable() {

        public void run() {
            mainPanel.treeAgent.clearLocalPlatform();
            }
        };
        SwingUtilities.invokeLater(resetIt);
    }
    */

    /**
     * Tells the Agent Tree to add a container.
     *
     * @param	cont name of the container to be added
     */

    public void addContainer(final String name, final InetAddress addr)
    {
        Runnable addIt = new Runnable()
        {
            public void run()
            {
                MutableTreeNode node = mainPanel.treeAgent.createNewNode(name, 0);
                mainPanel.treeAgent.addContainerNode((AgentTree.ContainerNode) node, "FIPACONTAINER", addr);
            }
        };
        SwingUtilities.invokeLater(addIt);
    }

    /**
     * Tells the Agent Tree to remove a specified container.
     *
     * @param	cont name of the container to be removed
     */

    public void removeContainer(final String name)
    {
        Runnable removeIt = new Runnable()
        {
            public void run()
            {
                mainPanel.treeAgent.removeContainerNode(name);
            }
        };
        SwingUtilities.invokeLater(removeIt);
    }

    /**
     * Tells the Agent Tree to add an agent.
     *
     * @param	cont name of the container to contain the new agent
     * @param name name of the agent to be created
     * @param addr address of the agent to be created
     * @param comm comment (usually type of the agent)
     */

    public void addAgent(final String containerName, final AID agentID)
    {
        Runnable addIt = new Runnable()
        {
            public void run()
            {
                String agentName = agentID.getName();
                // Hack: Get the first address if available,
                // cause AgentTree needs a String
                String agentAddress = null;
                if (agentID.getAddressesArray().length > 0)
                {
                    agentAddress = agentID.getAddressesArray()[0];
                }
                AgentTree.Node node = mainPanel.treeAgent.createNewNode(agentName, 1);
                mainPanel.treeAgent.addAgentNode((AgentTree.AgentNode) node, containerName, agentName, agentAddress, "FIPAAGENT");

            }
        };
        SwingUtilities.invokeLater(addIt);
    }

    /**
     * Tells the Agent Tree to remove a specified agent.
     *
     * @param	cont name of the container containing the agent
     * @param name name of the agent to be removed
     */

    public void removeAgent(final String containerName, final AID agentID)
    {
        Runnable removeIt = new Runnable()
        {
            public void run()
            {
                String agentName = agentID.getName();
                mainPanel.treeAgent.removeAgentNode(containerName, agentName);
            }
        };
        SwingUtilities.invokeLater(removeIt);
    }

    /**
     * Displays a dialog box with the error string.
     *
     * @param	errMsg error message to print
     */

    public void showError(String errMsg)
    {
        JOptionPane.showMessageDialog(null, errMsg, "Error in " + myLogger.getName(), JOptionPane.ERROR_MESSAGE);
    }

    public void showError(String errMsg, String title)
    {
        JOptionPane.showMessageDialog(null, errMsg, title, JOptionPane.ERROR_MESSAGE);
    }

    public Dimension getPreferredSize()
    {
        return new Dimension(700, 500);
    }

    public void addRemotePlatformFolder()
    {
        Runnable addIt = new Runnable()
        {
            public void run()
            {
                PopupMenuPlatform menu = new PopupMenuPlatform(actPro);
                mainPanel.treeAgent.register("REMOTEPLATFORMS", menu, "images/folderblue.gif");
                mainPanel.treeAgent.addRemotePlatformsFolderNode();

            }
        };
        SwingUtilities.invokeLater(addIt);
    }


    public void addRemotePlatform(AID name, APDescription profile)
    {

        final APDescription desc = profile;
        final AID ams = name;
        Runnable addIt = new Runnable()
        {

            public void run()
            {
                mainPanel.treeAgent.addRemotePlatformNode(ams, desc);

            }
        };
        SwingUtilities.invokeLater(addIt);
    }


    public void addRemoteAgentsToRemotePlatform(final APDescription platform, final Iterator i)
    {

        // Add an agent to a specified AMS
        Runnable addIt = new Runnable()
        {
            public void run()
            {

                while (i.hasNext())
                {
                    AMSAgentDescription agent = (AMSAgentDescription) i.next();

                    mainPanel.treeAgent.addRemoteAgentNode(agent, platform.getName());
                }
            }
        };
        SwingUtilities.invokeLater(addIt);

    }

    public void removeRemotePlatform(final String platformName)
    {

        Runnable addIt = new Runnable()
        {

            public void run()
            {
                mainPanel.treeAgent.removeRemotePlatformNode(platformName);

            }
        };
        SwingUtilities.invokeLater(addIt);
    }


    private void setUI(String ui)
    {
        try
        {
            UIManager.setLookAndFeel("javax.swing.plaf." + ui);
            SwingUtilities.updateComponentTreeUI(this);
            pack();
        }
        catch (Exception e)
        {
            System.out.println(e);
            e.printStackTrace(System.out);
        }
    }

    /**
     enables Motif L&F
     */

    public void setUI2Motif()
    {
        setUI("motif.MotifLookAndFeel");
    }

    /**
     enables Windows L&F
     */

    public void setUI2Windows()
    {
        setUI("windows.WindowsLookAndFeel");
    }

    /**
     enables Multi L&F
     */

    public void setUI2Multi()
    {
        setUI("multi.MultiLookAndFeel");
    }

    /**
     enables Metal L&F
     */
    public void setUI2Metal()
    {
        setUI("metal.MetalLookAndFeel");
    }

    /**
     * Provides async disposal of the gui to prevent deadlock when not running in
     * awt event dispatcher
     */

    public void disposeAsync()
    {

        class disposeIt implements Runnable
        {
            private Window toDispose;

            public disposeIt(Window w)
            {
                toDispose = w;
            }

            public void run()
            {
                toDispose.dispose();
            }
        }
        SwingUtilities.invokeLater(new disposeIt(this));
    }


}
